@@ -29,7 +29,7 @@ class DerivationTree:
         self.__value = value
         self.__children = None if children is None else tuple(children)
 
-        if id:
+        if id is not None:
             self._id = id
         else:
             self._id = DerivationTree.next_id
@@ -45,20 +45,23 @@ class DerivationTree:
             self.__is_open = True
 
     def __getstate__(self) -> bytes:
-        return zlib.compress(json.dumps(self.__dict__, default=lambda o: o.__dict__).encode('UTF-8'))
+        the_dict = self.__dict__
+        if '_DerivationTree__k_paths' in the_dict:
+            del the_dict['_DerivationTree__k_paths']
+        return zlib.compress(json.dumps(the_dict, default=lambda o: o.__dict__).encode('UTF-8'))
 
     def __setstate__(self, state: bytes):
         def from_dict(a_dict: dict) -> 'DerivationTree':
             result = DerivationTree.__new__(DerivationTree)
+            result.__k_paths = {}
 
             children_key = '_DerivationTree__children'
             ser_children = a_dict[children_key]
 
-            children = []
-            for child in ser_children:
-                children.append(from_dict(child))
-
-            a_dict[children_key] = children
+            if ser_children is None:
+                a_dict[children_key] = None
+            else:
+                a_dict[children_key] = tuple([from_dict(child) for child in ser_children])
 
             result.__dict__.update(a_dict)
 
@@ -144,10 +147,13 @@ class DerivationTree:
         return not self.is_open()
 
     @lru_cache(maxsize=20)
-    def get_subtree(self, path: Path) -> 'DerivationTree':
+    def get_subtree(self, path: Path) -> Optional['DerivationTree']:
         """Access a subtree based on `path` (a list of children numbers)"""
         curr_node = self
         while path:
+            if not curr_node.children:
+                return None
+
             curr_node = curr_node.children[path[0]]
             path = path[1:]
 
@@ -201,7 +207,16 @@ class DerivationTree:
         return result
 
     def find_node(self, node_or_id: Union['DerivationTree', int]) -> Optional[Path]:
-        """Finds a node by its (assumed unique) ID. Returns the path relative to this node."""
+        """
+        Finds a node by its (assumed unique) ID. Returns the path relative to this node.
+
+        Attention: Might return an empty tuple, which indicates that the searched-for node
+        is the root of the tree! Don't use as in `if not find_node(...).`, use
+        `if find_node(...) is not None:`.
+
+        :param node_or_id: The node or node ID to search for.
+        :return: The path to the node or None.
+        """
         if isinstance(node_or_id, DerivationTree):
             node_or_id = node_or_id.id
 
@@ -210,17 +225,6 @@ class DerivationTree:
         except StopIteration:
             return None
 
-        stack: List[Tuple[Path, DerivationTree]] = [((), self)]
-        while stack:
-            path, tree = stack.pop()
-            if tree.id == node_or_id:
-                return path
-
-            if tree.children:
-                stack.extend([(path + (idx,), child) for idx, child in enumerate(tree.children)])
-
-        return None
-
     def traverse(
             self,
             action: Callable[[Path, 'DerivationTree'], None],
@@ -439,8 +443,7 @@ class DerivationTree:
 
         result = self
         for tree_id in id_subst_map:
-            path = result.find_node(tree_id)
-            if path is not None:
+            if (path := result.find_node(tree_id)) is not None:
                 result = result.replace_path(path, id_subst_map[tree_id])
 
         return result
@@ -468,25 +471,9 @@ class DerivationTree:
                    for idx, _ in enumerate(self.children))
 
     def is_potential_prefix(self, other: 'DerivationTree') -> bool:
-        """Returns `True` iff this the `other` tree can be extended such that this tree
-        is a prefix of the `other` tree."""
-        if self.value != other.value:
-            return False
-
-        if not self.children:
-            return self.children is None or (not other.children and other.children is not None)
-
-        if not other.children:
-            return other.children is None
-
-        assert self.children
-        assert other.children
-
-        if len(self.children) != len(other.children):
-            return False
-
-        return all(self.children[idx].is_potential_prefix(other.children[idx])
-                   for idx, _ in enumerate(self.children))
+        # It's a potential prefix if for all common paths of the two trees, the leaves are equal.
+        common_paths = {path for path, _ in other.paths()}.intersection({path for path, _ in self.paths()})
+        return all(self.get_subtree(path).value == other.get_subtree(path).value for path in common_paths)
 
     @staticmethod
     def from_parse_tree(tree: ParseTree):
