@@ -10,7 +10,7 @@ from typing import Set, Generator, Tuple, List, Dict, Union, TypeVar, Sequence,
 
 import datrie
 
-from isla.type_defs import Path, Grammar, ParseTree, ImmutableGrammar, CanonicalGrammar
+from isla.type_defs import Path, Grammar, ParseTree, ImmutableGrammar, CanonicalGrammar, ImmutableList
 
 S = TypeVar('S')
 T = TypeVar('T')
@@ -194,9 +194,11 @@ def weighted_geometric_mean(seq: Sequence[float], weights: Sequence[float]) -> f
 
 
 def grammar_to_immutable(grammar: Grammar) -> ImmutableGrammar:
-    return cast(
-        Tuple[Tuple[str, Tuple[str, ...]]],
-        tuple({k: tuple(v) for k, v in grammar.items()}.items()))
+    return cast(ImmutableGrammar, tuple({k: tuple(v) for k, v in grammar.items()}.items()))
+
+
+def grammar_to_mutable(grammar: ImmutableGrammar) -> Grammar:
+    return {nonterminal: list(expansion) for nonterminal, expansion in grammar}
 
 
 def immutable_to_grammar(immutable: ImmutableGrammar) -> Grammar:
@@ -655,3 +657,7 @@ def nth_occ(haystack: Sequence[T], needle: T, n: int) -> Optional[int]:
             num_occs += 1
 
     return None
+
+
+def list_set(ilist: ImmutableList[T], repl_idx: int, new_elem: T) -> ImmutableList[T]:
+    return tuple([new_elem if idx == repl_idx else elem for idx, elem in enumerate(ilist)])
